#define _XTAL_FREQ 4000000
#include <htc.h>

__CONFIG(FOSC_XT & WDTE_OFF & PWRTE_ON & CP_OFF & BOREN_OFF & LVP_OFF & CPD_OFF);

// Precaricamento dati nella EEPROM
// (TIMER_MIN, TIMER_SEC, CALIB_SEC, CALIB_PAS, NULL, NULL, NULL, NULL);  
__EEPROM_DATA(01,00, 10, 12, 0,0,0,0);

// Contatore di clock del TMR0
char TMR0_CK=0;

// Indicatore asincrono di aggiornamento timer
bit AGG_TIMER;

/***** DRIVER LCD ****/
#include "HD44780.h"
#include "HD44780.c"

/* Driver bromografo */
#include "Bromografo.h"
#include "Bromografo.c"

// Elementi del menu principale
const char * M_Pri[] = {
	"AVVIA ESPOSIZ.\n",
	"AVVIA CALIBRA.\n",
	"IMPOSTAZIONI   "
};

// Elementi del menu Impostazioni
const char * M_Imp[] = {
	"IMPOSTA CALIBR.\n",
	"IMPOSTA TIMER"
};

void main(){
	// Inizializzazione dei pin come I o O
	TRISA=0x20; // { NC,  NC,  IN, OUT, OUT, OUT, OUT, OUT}
	TRISB=0x00; // {OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT}
	TRISC=0x0B; // {OUT, OUT, OUT, OUT,  IN, OUT,  IN,  IN}
	PORTA=0x00;
	PORTB=0x00;
	PORTC=0x00;
	
	// PORTA: tutti pin hanno funzione I/O digitale
	ADCON1=0x06;
	
	// TMR0 Prescaler (PS2|PS1|PS0): 1|0|0 -> 1:32
	OPTION_REG=0x84;
	
	// Aggiornamento timer inizialmente disabilitato
	// Verrà abilitato nella routine T0ISR
	AGG_TIMER = 0;
	
	// LED di stato: ON (pronto)
	B_LED_STATO = 1;
	
	// LED UV: spenti
	B_COM_UV1 = 0;
	B_COM_UV2 = 0;
	
	// Inizializzazione del display LCD
	LCD_Initialize();
	
	// Variabile di menu
	char COM;
	
	while(1){
		/* MENU PRINCIPALE */

		// Visualizzazione del menu principale
		COM = VisualizzaMenu(M_Pri, 3);
		switch (COM){
			case 0: {
				// Avvia esposizione
				COM = Esposizione(eeprom_read(B_TIMER_MIN), eeprom_read(B_TIMER_SEC));
				
				if (COM == 0)
					LCD_WriteString("  Esposizione\n   terminata!");
				else
					LCD_WriteString("  Esposizione\n   annullata!");
				
				break;
			}
			
			case 1:{
				// Avvia calibrazione
				COM = Calibrazione();
				
				if (COM == 0)
					LCD_WriteString("  Calibrazione\n   terminata!");
				else
					LCD_WriteString("  Calibrazione\n   annullata!");
				
				break;
			}
			
			case 2:{
				/* IMPOSTAZIONI */
				
				// Visualizzazione del menu impostazioni
				COM = VisualizzaMenu(M_Imp, 2);
				switch (COM){
					case 0: {
						// Imposta calibrazione
						ImpostaCalibrazione();
						
						break;
					}
					
					case 1: {
						// Imposta timer
						ImpostaTimer(B_TIMER_MIN, B_TIMER_SEC);
						
						break;
					}
				}
				
				break;
			}
		}
		
		// Tempo di visualizzazione degli
		// eventuali messaggi di uscita
		__delay_ms(1500);
	}
}

// Routine di gestione degli interrupt
// Verrà usata solo per l'interrupt del TMR0
void interrupt T0ISR(){
	if (T0IF){
		TMR0_CK++;
		
		if (TMR0_CK == 124){
			TMR0_CK = 0;
			AGG_TIMER=1; // indica che il timer sul display va aggiornato
		}
		
		T0IF=0;
	}
}
